__author__ = "Richard James"
__copyright__ = "Copyright 2012, Richard James"
__license__ = "GPL 3"
__version__ = "0.0.0"
__maintainer__ = "Richard James"
__email__ = "richardjam13@gmail.com"
__status__ = "Development"

import pygame
from pygame.locals import *
from random import randint
from pkg_resources import resource_stream, resource_string, resource_filename
import ConfigParser
import sys
from xml.dom.minidom import parse

debug = False

class State():
    rows = []

    def load(self, level_number):
        self.rows = []
        levels_filename = resource_filename(__name__, 'data/levels.txt')
        dom = parse(levels_filename)
        levels=dom.getElementsByTagName('level')
        for level in levels:
            if debug: print (level.getAttribute('number'))
            # find the right level
            if level.getAttribute('number') == str(level_number):
                # extract the rows
                for rows in level.getElementsByTagName('row'):
                    if debug: print ("["+rows.firstChild.data+"]")
                    self.rows.append(rows.firstChild.data)
                # TODO extract the entities

                break




class SoundHelper():
    sounds = {}

    def get_sound(self, name):
        sound_key = name
        if sound_key not in self.sounds.keys():
            sound = pygame.mixer.Sound(resource_stream(__name__, name))
            self.sounds[sound_key] = sound
            return sound
        else:
            return self.sounds[sound_key]


class FontHelper():
    fonts = {}
    screen = None

    def __init__(self, screen):
        self.screen = screen

    def get_font(self, name, weight):
        font_key = name + str(weight)
        if font_key not in self.fonts.keys():
            font = pygame.font.Font(name, weight)
            self.fonts[font_key] = font
            return font
        else:
            return self.fonts[font_key]

class ImageHelper():
    images = {}

    def get_image(self, name):
        image_key = name
        if image_key not in self.images.keys():
            image = pygame.image.load(resource_stream(__name__, name))
            self.images[image_key] = image
            return image
        else:
            return self.images[image_key]


class SpriteEntity():
    res = None
    x = 0
    y = 0
    direction = "LEFT"
    frame = 0

    def __init__(self, resource):
        self.res = resource


class Player(SpriteEntity):

    def move_left(self):
        self.x -= 1
        if self.direction == "LEFT": self.cycle_frame()
        else:
            self.direction = "LEFT"
            self.frame = 0
    def move_right(self):
        self.x += 1
        if self.direction == "RIGHT": self.cycle_frame()
        else:
            self.direction = "RIGHT"
            self.frame = 0
    def move_up(self):
        self.y -= 1
        if self.direction == "UP": self.cycle_frame()
        else:
            self.direction = "UP"
            self.frame = 0
    def move_down(self):
        self.y += 1
        if self.direction == "DOWN": self.cycle_frame()
        else:
            self.direction = "DOWN"
            self.frame = 0

    def cycle_frame(self):
        if self.frame == 0: self.frame = 1
        elif self.frame == 1: self.frame = 0

    def draw(self):
        font = self.res.font_helper.get_font("freesansbold.ttf", 16)
        sprite = self.res.image_helper.get_image('data/monster.png')

        if self.direction == "RIGHT": frame_y = 0
        if self.direction == "LEFT": frame_y = 1
        if self.direction == "UP": frame_y = 2
        if self.direction == "DOWN": frame_y = 3

        frame_x = self.frame

        msg = str((self.x-self.res.x_offset)/self.res.tile_size_x) + ":" + str((self.y-self.res.y_offset)/self.res.tile_size_y) + '-'
        msg += str((self.x-self.res.x_offset+self.res.tile_size_x-1)/self.res.tile_size_x) + ":" + str((self.y-self.res.y_offset+self.res.tile_size_y-1)/self.res.tile_size_y)
        msgSurf = font.render(msg, False, pygame.Color('yellow'))
        msgRect = msgSurf.get_rect()
        msgRect.topleft = (10,20)
        self.res.screen.blit(msgSurf,msgRect)
        self.res.screen.blit(sprite, (self.x, self.y),(frame_x*self.res.tile_size_x,frame_y*self.res.tile_size_y,self.res.tile_size_x,self.res.tile_size_y))

class Beaker(SpriteEntity):
    def draw(self):
        pass


class Robot(SpriteEntity):

    def draw(self):
        pass

class SpriteHolder():
    sprites = {}
    res = None
    sta = None
    def __init__(self, resource, state):
        self.res = resource
        self.sta = state

    def init_from_level(self):
        # TODO pull entity positions from level data and make sprites from them
        self.player = Player(self.res)
        pass

    def draw_sprites(self):
        # TODO loop through all sprites in the dictionary and draw them
        pass

    def get_sound(self, name):
        sound_key = name
        if sound_key not in self.sounds.keys():
            sound = pygame.mixer.Sound(resource_stream(__name__, name))
            self.sounds[sound_key] = sound
            return sound
        else:
            return self.sounds[sound_key]

class Resources():
    font_helper = None
    sound_helper = None
    image_helper = None
    screen = None

    x_offset = 16
    y_offset = 0
    tile_size_x = 48
    tile_size_y = 48


    def init_graphics(self, x, y, title):
        pygame.init()
        self.screen = pygame.display.set_mode((x, y))
        pygame.display.set_caption(title)
        self.font_helper = FontHelper(self.screen)
        self.sound_helper = SoundHelper()
        self.image_helper = ImageHelper()

    def draw_level(self, state):
        sprite = self.image_helper.get_image('data/tiles.png')

        x = 0
        y = 0

        sx = 1
        sy = 0
        self.screen.fill(pygame.Color('black'))
        for row in state.rows:
            line = ""
            for tile in row:
                sx = int(tile)
                source = (sx * self.tile_size_x, sy * self.tile_size_y, self.tile_size_x, self.tile_size_y)
                dest = (self.x_offset + x * self.tile_size_x, self.y_offset + y * self.tile_size_y)
                self.screen.blit(sprite, dest , source)
                line = line + tile
                x += 1
            x = 0
            y += 1
            if debug: print (line)


        if debug:

            for x in range(0, 17):
                pygame.draw.line(self.screen, pygame.Color('red'),(self.x_offset+x*self.tile_size_x,0),(self.x_offset+x*self.tile_size_x,512))
            for y in range(0, 11):
                pygame.draw.line(self.screen, pygame.Color('red'),(0,self.y_offset+y*self.tile_size_y),(800,self.y_offset+y*self.tile_size_y))



class Game():
    res = None
    sta = None

    def main(self):
        self.sta = State()
        self.res = Resources()
        self.sta.load(1)
        self.res.init_graphics(800, 500, 'Lab Escape')
        pygame.key.set_repeat(2, 2)
        self.player = Player(self.res)
        self.player.x = 16+48
        self.player.y = 48
        self.fps_clock = pygame.time.Clock()

        pygame.display.update()

        self.event_loop()


    def event_loop(self):

        while True:
            for event in pygame.event.get():
                if event.type == QUIT:
                    pygame.quit()
                    sys.exit()
                if event.type == KEYDOWN:
                    if event.key == K_ESCAPE:
                        pygame.quit()
                        sys.exit()
                    if event.key == K_LEFT:
                        self.player.move_left()
                    if event.key == K_RIGHT:
                        self.player.move_right()
                    if event.key == K_UP:
                        self.player.move_up()
                    if event.key == K_DOWN:
                        self.player.move_down()


            self.res.draw_level(self.sta)
            self.player.draw()
            pygame.display.update()
            self.fps_clock.tick(30)

def main():
    game = Game()
    game.main()

if __name__ == '__main__':
    main()

