__author__ = "Richard James"
__copyright__ = "Copyright 2012, Richard James"
__license__ = "GPL 3"
__version__ = "0.0.0"
__maintainer__ = "Richard James"
__email__ = "richardjam13@gmail.com"
__status__ = "Development"

from pkg_resources import resource_stream, resource_string, resource_filename
import ConfigParser
import sys
from xml.etree.ElementTree import Element, SubElement, Comment, tostring
from xml.etree import ElementTree
from xml.dom import minidom

def create_default_config_file():
    config = ConfigParser.RawConfigParser()

    # When adding sections or items, add them in the reverse order of
    # how you want them to be displayed in the actual file.
    # In addition, please note that using RawConfigParser's and the raw
    # mode of ConfigParser's respective set functions, you can assign
    # non-string values to keys internally, but will receive an error
    # when attempting to write to a file or when you get it in non-raw
    # mode. SafeConfigParser does not allow such assignments to take place.
    config.add_section('Keys')
    config.set('Keys', 'up', 'K_a')
    config.set('Keys', 'down', 'K_s')
    config.set('Keys', 'left', 'K_a')
    config.set('Keys', 'right', 'K_d')
    config.set('Keys', 'quit', 'K_ESCAPE')
    config.set('Keys', 'activate', 'K_SPACE')

    config.add_section('Settings')
    config.set('Settings', 'fullscreen', 'false')

    with open((resource_filename(__name__, 'config.txt')), 'wb') as configfile:
        config.write(configfile)


def prettify(elem):
    """Return a pretty-printed XML string for the Element.
    """
    rough_string = ElementTree.tostring(elem, 'utf-8')
    reparsed = minidom.parseString(rough_string)
    return reparsed.toprettyxml()

def create_default_level_file():
    levels_filename = resource_filename(__name__, 'data/levels.txt')
    script_file = resource_filename(__name__, 'data/scripts.txt')

    top = Element('levels')

    #comment = Comment('')
    #top.append(comment)
    for l in range(0,11):

        level = SubElement(top,'level', {'number':str(l+1)})
        row = SubElement(level,'row', {'number':'1'})
        #          '################'
        row.text = '1111111111111111'
        row = SubElement(level,'row', {'number':'2'})
        #          '################'
        row.text = '1000000000000001'
        row = SubElement(level,'row', {'number':'3'})
        #          '################'
        row.text = '1000000000000001'
        row = SubElement(level,'row', {'number':'4'})
        #          '################'
        row.text = '1000000000000001'
        row = SubElement(level,'row', {'number':'5'})
        #          '################'
        row.text = '1000000000000001'
        row = SubElement(level,'row', {'number':'6'})
        #          '################'
        row.text = '1000000000000001'
        row = SubElement(level,'row', {'number':'7'})
        #          '################'
        row.text = '1000000000000001'
        row = SubElement(level,'row', {'number':'8'})
        #          '################'
        row.text = '1000000000000001'
        row = SubElement(level,'row', {'number':'9'})
        #          '################'
        row.text = '1000000000000001'
        row = SubElement(level,'row', {'number':'10'})
        #          '################'
        row.text = '1111111111111111'
        entity = SubElement(level,'entity', {'type':'beaker','x':'2','y':'4'})
        entity.text = "Beaker1"
        entity = SubElement(level,'entity', {'type':'robot','x':'3','y':'3','dir':'up'})
        entity.text = "Robot1"
        entity = SubElement(level,'entity', {'type':'exit','x':'5','y':'7'})
        entity.text = "Exit1"

    print (tostring(top))
    with open(levels_filename , "w") as f:
        f.write(tostring(top))
        #f.write( prettify(top) )
        f.close()


def main():
    create_default_config_file()
    create_default_level_file()

if __name__ == '__main__':
    main()